﻿using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Channels;
using System.Xml;
using System.Net;
using System.IO;
using System.ServiceModel.Configuration;
using System.Configuration;
using System.Web.Script.Serialization;

namespace AppFx.BZ.Adapter.Behaviours.AzureAD
{
    public class AzureADSecurityBehavior : IClientMessageInspector, IEndpointBehavior
    {
        private string _resourceId;
        private string _password;
        private string _clientId;
        private string _username;

        public AzureADSecurityBehavior(string username, string password, string resourceId, string clientId)
        {
            _username = username;
            _password = password;
            _resourceId = resourceId;
            _clientId = clientId;
        }
        
        #region IClientMessageInspector

        public void AfterReceiveReply(ref System.ServiceModel.Channels.Message reply, object correlationState)
        {
            // do nothing
        }
        public object BeforeSendRequest(ref System.ServiceModel.Channels.Message request, System.ServiceModel.IClientChannel channel)
        {           
            HttpRequestMessageProperty httpRequest = null;
            if (request.Properties.ContainsKey(HttpRequestMessageProperty.Name))
            {
                httpRequest = request.Properties[HttpRequestMessageProperty.Name] as HttpRequestMessageProperty;

                var tokenManager = new TokenManager();
                tokenManager.ClientId = _clientId;
                tokenManager.Password = _password;
                tokenManager.ResourceId = _resourceId;
                tokenManager.Username = _username;
                string token = tokenManager.AccessToken();

                WebHeaderCollection headers = httpRequest.Headers;                
                headers.Add(HttpRequestHeader.Authorization, "Bearer " + token);
            }
            else
            {

            }            
            return null;
        }

        #endregion IClientMessageInspector

        #region IEndpointBehavior

        public void AddBindingParameters(ServiceEndpoint endpoint, System.ServiceModel.Channels.BindingParameterCollection bindingParameters)
        {
            // do nothing
        }

        public void ApplyClientBehavior(ServiceEndpoint endpoint, ClientRuntime clientRuntime)
        {
            clientRuntime.MessageInspectors.Add(this);
        }

        public void ApplyDispatchBehavior(ServiceEndpoint endpoint, EndpointDispatcher endpointDispatcher)
        {
            // do nothing
        }

        public void Validate(ServiceEndpoint endpoint)
        {
            // do nothing
        }

        #endregion IEndpointBehavior
    }
}
